@@ -68,7 +68,6 @@ class GVDialer(object):
 	the functions include login, setting up a callback number, and initalting a callback
 	"""
 
-	_isNotLoginPageRe = re.compile(r"""I cannot access my account""")
 	_tokenRe = re.compile(r"""<input.*?name="_rnr_se".*?value="(.*?)"\s*/>""")
 	_accountNumRe = re.compile(r"""<b class="ms2">(.{14})</b></div>""")
 	_callbackRe = re.compile(r"""\s+(.*?):\s*(.*?)<br\s*/>\s*$""", re.M)
@@ -88,7 +87,6 @@ class GVDialer(object):
 	_accountNumberURL = "https://www.google.com/voice/mobile"
 	_forwardURL = "https://www.google.com/voice/mobile/phones"
 
-	_inboxURL = "https://www.google.com/voice/inbox/"
 	_recentCallsURL = "https://www.google.com/voice/inbox/recent/"
 	_placedCallsURL = "https://www.google.com/voice/inbox/recent/placed/"
 	_receivedCallsURL = "https://www.google.com/voice/inbox/recent/received/"
@@ -103,9 +101,9 @@ class GVDialer(object):
 		if os.path.isfile(cookieFile):
 			self._browser.cookies.load()
 
+		self._token = ""
 		self._accountNum = None
 		self._lastAuthed = 0.0
-		self._token = ""
 		self._callbackNumber = ""
 		self._callbackNumbers = {}
 
@@ -122,16 +120,12 @@ class GVDialer(object):
 			return True
 
 		try:
-			inboxPage = self._browser.download(self._inboxURL)
-		except urllib2.URLError, e:
+			self._grab_account_info()
+		except StandardError, e:
 			warnings.warn(traceback.format_exc())
-			raise RuntimeError("%s is not accesible" % self._inboxURL)
-
-		self._browser.cookies.save()
-		if self._isNotLoginPageRe.search(inboxPage) is not None:
 			return False
 
-		self._grab_account_info()
+		self._browser.cookies.save()
 		self._lastAuthed = time.time()
 		return True
 
@@ -147,6 +141,9 @@ class GVDialer(object):
 			'Email' : username,
 			'Passwd' : password,
 			'service': "grandcentral",
+			"ltmpl": "mobile",
+			"btmpl": "mobile",
+			"PersistentCookie": "yes",
 		})
 
 		try:
